@@ -1,5 +1,5 @@
 /* Tcl/Tk command definitions for Insight - Stack.
-   Copyright 2001 Free Software Foundation, Inc.
+   Copyright 2001, 2002 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -19,8 +19,6 @@
    Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
-#include "frame.h"
-#include "value.h"
 #include "target.h"
 #include "breakpoint.h"
 #include "linespec.h"
@@ -86,21 +84,17 @@ Gdbtk_Stack_Init (Tcl_Interp *interp)
  *    All variables defined in the given block.
  */
 static int
-gdb_block_vars (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_block_vars (ClientData clientData, Tcl_Interp *interp,
+		int objc, Tcl_Obj *CONST objv[])
 {
   struct block *block;
-  int nsyms, i;
+  int i;
   struct symbol *sym;
   CORE_ADDR start, end;
 
   if (objc < 3)
     {
       Tcl_WrongNumArgs (interp, 1, objv, "startAddr endAddr");
-      result_ptr->flags |= GDBTK_IN_TCL_RESULT;
       return TCL_ERROR;
     }
 
@@ -108,8 +102,8 @@ gdb_block_vars (clientData, interp, objc, objv)
   if (selected_frame == NULL)
     return TCL_OK;
 
-  start = parse_and_eval_address (Tcl_GetStringFromObj (objv[1], NULL));
-  end   = parse_and_eval_address (Tcl_GetStringFromObj (objv[2], NULL));
+  start = string_to_core_addr (Tcl_GetStringFromObj (objv[1], NULL));
+  end   = string_to_core_addr (Tcl_GetStringFromObj (objv[2], NULL));
   
   block = get_frame_block (selected_frame);
 
@@ -117,10 +111,8 @@ gdb_block_vars (clientData, interp, objc, objv)
     {
       if (BLOCK_START (block) == start && BLOCK_END (block) == end)
 	{
-	  nsyms = BLOCK_NSYMS (block);
-	  for (i = 0; i < nsyms; i++)
+	  ALL_BLOCK_SYMBOLS (block, i, sym)
 	    {
-	      sym = BLOCK_SYM (block, i);
 	      switch (SYMBOL_CLASS (sym))
 		{
 		case LOC_ARG:		  /* argument              */
@@ -165,14 +157,11 @@ gdb_block_vars (clientData, interp, objc, objv)
  *    A list of all valid blocks in the selected_frame.
  */
 static int
-gdb_get_blocks (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_blocks (ClientData clientData, Tcl_Interp *interp,
+		int objc, Tcl_Obj *CONST objv[])
 {
   struct block *block;
-  int nsyms, i, junk;
+  int i, junk;
   struct symbol *sym;
   CORE_ADDR pc;
 
@@ -184,11 +173,9 @@ gdb_get_blocks (clientData, interp, objc, objv)
       pc = get_frame_pc (selected_frame);
       while (block != 0)
 	{
-	  nsyms = BLOCK_NSYMS (block);
 	  junk = 0;
-	  for (i = 0; i < nsyms; i++)
+	  ALL_BLOCK_SYMBOLS (block, i, sym)
 	    {
-	      sym = BLOCK_SYM (block, i);
 	      switch (SYMBOL_CLASS (sym))
 		{
 		default:
@@ -255,53 +242,44 @@ gdb_get_blocks (clientData, interp, objc, objv)
  * of the command, because the call wrapper uses this instead...
  */
 static int
-gdb_get_args_command (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_args_command (ClientData clientData, Tcl_Interp *interp,
+		      int objc, Tcl_Obj *CONST objv[])
 {
   return gdb_get_vars_command ((ClientData) 1, interp, objc, objv);
 }
 
 
 static int
-gdb_get_locals_command (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_locals_command (ClientData clientData, Tcl_Interp *interp,
+			int objc, Tcl_Obj *CONST objv[])
 {
   return gdb_get_vars_command ((ClientData) 0, interp, objc, objv);
 }
 
 /* This implements the tcl commands "gdb_get_locals" and "gdb_get_args"
 
- * This function sets the Tcl interpreter's result to a list of variable names
- * depending on clientData. If clientData is one, the result is a list of
- * arguments; zero returns a list of locals -- all relative to the block
- * specified as an argument to the command. Valid commands include
- * anything decode_line_1 can handle (like "main.c:2", "*0x02020202",
- * and "main").
- *
- * Tcl Arguments:
- *   linespec - the linespec defining the scope of the lookup. Empty string
- *              to use the current block in the innermost frame.
- * Tcl Result:
- *   A list of the locals or args
- */
+* This function sets the Tcl interpreter's result to a list of variable names
+* depending on clientData. If clientData is one, the result is a list of
+* arguments; zero returns a list of locals -- all relative to the block
+* specified as an argument to the command. Valid commands include
+* anything decode_line_1 can handle (like "main.c:2", "*0x02020202",
+* and "main").
+*
+* Tcl Arguments:
+*   linespec - the linespec defining the scope of the lookup. Empty string
+*              to use the current block in the innermost frame.
+* Tcl Result:
+*   A list of the locals or args
+*/
 static int
-gdb_get_vars_command (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_vars_command (ClientData clientData, Tcl_Interp *interp,
+		      int objc, Tcl_Obj *CONST objv[])
 {
   struct symtabs_and_lines sals;
   struct symbol *sym;
   struct block *block;
   char **canonical, *args;
-  int i, nsyms, arguments;
+  int i, arguments;
 
   if (objc > 2)
     {
@@ -322,8 +300,7 @@ gdb_get_vars_command (clientData, interp, objc, objv)
       sals = decode_line_1 (&args, 1, NULL, 0, &canonical);
       if (sals.nelts == 0)
 	{
-	  Tcl_SetStringObj (result_ptr->obj_ptr,
-			    "error decoding line", -1);
+	  gdbtk_set_result (interp, "error decoding line");
 	  return TCL_ERROR;
 	}
 
@@ -344,10 +321,8 @@ gdb_get_vars_command (clientData, interp, objc, objv)
 
   while (block != 0)
     {
-      nsyms = BLOCK_NSYMS (block);
-      for (i = 0; i < nsyms; i++)
+      ALL_BLOCK_SYMBOLS (block, i, sym)
 	{
-	  sym = BLOCK_SYM (block, i);
 	  switch (SYMBOL_CLASS (sym))
 	    {
 	    default:
@@ -368,7 +343,7 @@ gdb_get_vars_command (clientData, interp, objc, objv)
 	    case LOC_BASEREG_ARG:	/* basereg arg           */
 	      if (arguments)
 		Tcl_ListObjAppendElement (interp, result_ptr->obj_ptr,
-				  Tcl_NewStringObj (SYMBOL_NAME (sym), -1));
+					  Tcl_NewStringObj (SYMBOL_NAME (sym), -1));
 	      break;
 	    case LOC_LOCAL:	/* stack local           */
 	    case LOC_BASEREG:	/* basereg local         */
@@ -376,7 +351,7 @@ gdb_get_vars_command (clientData, interp, objc, objv)
 	    case LOC_REGISTER:	/* register              */
 	      if (!arguments)
 		Tcl_ListObjAppendElement (interp, result_ptr->obj_ptr,
-				  Tcl_NewStringObj (SYMBOL_NAME (sym), -1));
+					  Tcl_NewStringObj (SYMBOL_NAME (sym), -1));
 	      break;
 	    }
 	}
@@ -400,11 +375,8 @@ gdb_get_vars_command (clientData, interp, objc, objv)
  *    The currently selected block's start and end addresses
  */
 static int
-gdb_selected_block (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_selected_block (ClientData clientData, Tcl_Interp *interp,
+		    int objc, Tcl_Obj *CONST objv[])
 {
   char *start = NULL;
   char *end   = NULL;
@@ -435,20 +407,17 @@ gdb_selected_block (clientData, interp, objc, objv)
 
 /* This implements the tcl command gdb_selected_frame
 
- * Returns the address of the selected frame
- * frame.
- *
- * Arguments:
- *    None
- * Tcl Result:
- *    The currently selected frame's address
- */
+* Returns the address of the selected frame
+* frame.
+*
+* Arguments:
+*    None
+* Tcl Result:
+*    The currently selected frame's address
+*/
 static int
-gdb_selected_frame (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_selected_frame (ClientData clientData, Tcl_Interp *interp,
+		    int objc, Tcl_Obj *CONST objv[])
 {
   char *frame;
 
@@ -473,18 +442,14 @@ gdb_selected_frame (clientData, interp, objc, objv)
  *    A list of function names
  */
 static int
-gdb_stack (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_stack (ClientData clientData, Tcl_Interp *interp,
+	   int objc, Tcl_Obj *CONST objv[])
 {
   int start, count;
 
   if (objc < 3)
     {
       Tcl_WrongNumArgs (interp, 1, objv, "start count");
-      result_ptr->flags |= GDBTK_IN_TCL_RESULT;
       return TCL_ERROR;
     }
 
