@@ -5,19 +5,26 @@
  *	that didn't fit in any particular file of the toolkit.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
- * Copyright (c) 1998 by Scriptics Corporation.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCmds.c,v 1.8 1999/01/26 04:11:20 jingham Exp $
+ * RCS: @(#) $Id: tkCmds.c,v 1.7.6.2 2000/09/26 16:07:54 spolk Exp $
  */
 
 #include "tkPort.h"
 #include "tkInt.h"
 #include <errno.h>
 
+#if defined(__WIN32__)
+#include "tkWinInt.h"
+#elif defined(MAC_TCL)
+#include "tkMacInt.h"
+#else
+#include "tkUnixInt.h"
+#endif
+
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -55,12 +62,10 @@ Tk_BellObjCmd(clientData, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
+    static char *bellOptions[] = {"-displayof", (char *) NULL};
     Tk_Window tkwin = (Tk_Window) clientData;
+    char *displayName;
     int index;
-    char *string;
-    static char *optionStrings[] = {
-	"-displayof",	NULL
-    };
 
     if ((objc != 1) && (objc != 3)) {
 	Tcl_WrongNumArgs(interp, 1, objv, "?-displayof window?");
@@ -68,12 +73,13 @@ Tk_BellObjCmd(clientData, interp, objc, objv)
     }
 
     if (objc == 3) {
-	if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, "option", 0,
+	if (Tcl_GetIndexFromObj(interp, objv[1], bellOptions, "option", 0,
 		&index) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	string = Tcl_GetStringFromObj(objv[2], NULL);
-	tkwin = Tk_NameToWindow(interp, string, tkwin);
+	displayName = Tcl_GetStringFromObj(objv[2], (int *) NULL);
+	
+	tkwin = Tk_NameToWindow(interp, displayName, tkwin);
 	if (tkwin == NULL) {
 	    return TCL_ERROR;
 	}
@@ -154,7 +160,7 @@ Tk_BindCmd(clientData, interp, argc, argv)
 	    Tcl_ResetResult(interp);
 	    return TCL_OK;
 	}
-	interp->result = command;
+	Tcl_SetResult(interp, command, TCL_STATIC);
     } else {
 	Tk_GetAllBindings(interp, winPtr->mainPtr->bindingTable, object);
     }
@@ -186,7 +192,6 @@ TkBindEventProc(winPtr, eventPtr)
 {
 #define MAX_OBJS 20
     ClientData objects[MAX_OBJS], *objPtr;
-    static Tk_Uid allUid = NULL;
     TkWindow *topLevPtr;
     int i, count;
     char *p;
@@ -234,10 +239,7 @@ TkBindEventProc(winPtr, eventPtr)
 	} else {
 	    count = 3;
 	}
-	if (allUid == NULL) {
-	    allUid = Tk_GetUid("all");
-	}
-	objPtr[count-1] = (ClientData) allUid;
+	objPtr[count-1] = (ClientData) Tk_GetUid("all");
     }
     Tk_BindEvent(winPtr->mainPtr->bindingTable, eventPtr, (Tk_Window) winPtr,
 	    count, objPtr);
@@ -383,7 +385,7 @@ TkFreeBindingTags(winPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_DestroyCmd --
+ * Tk_DestroyObjCmd --
  *
  *	This procedure is invoked to process the "destroy" Tcl command.
  *	See the user documentation for details on what it does.
@@ -398,19 +400,19 @@ TkFreeBindingTags(winPtr)
  */
 
 int
-Tk_DestroyCmd(clientData, interp, argc, argv)
+Tk_DestroyObjCmd(clientData, interp, objc, objv)
     ClientData clientData;		/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Tk_Window window;
     Tk_Window tkwin = (Tk_Window) clientData;
     int i;
 
-    for (i = 1; i < argc; i++) {
-	window = Tk_NameToWindow(interp, argv[i], tkwin);
+    for (i = 1; i < objc; i++) {
+	window = Tk_NameToWindow(interp, Tcl_GetString(objv[i]), tkwin);
 	if (window == NULL) {
 	    Tcl_ResetResult(interp);
 	    continue;
@@ -432,7 +434,7 @@ Tk_DestroyCmd(clientData, interp, argc, argv)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_LowerCmd --
+ * Tk_LowerObjCmd --
  *
  *	This procedure is invoked to process the "lower" Tcl command.
  *	See the user documentation for details on what it does.
@@ -448,37 +450,37 @@ Tk_DestroyCmd(clientData, interp, argc, argv)
 
 	/* ARGSUSED */
 int
-Tk_LowerCmd(clientData, interp, argc, argv)
+Tk_LowerObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Tk_Window mainwin = (Tk_Window) clientData;
     Tk_Window tkwin, other;
 
-    if ((argc != 2) && (argc != 3)) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " window ?belowThis?\"", (char *) NULL);
+    if ((objc != 2) && (objc != 3)) {
+	Tcl_WrongNumArgs(interp, 1, objv, "window ?belowThis?");
 	return TCL_ERROR;
     }
 
-    tkwin = Tk_NameToWindow(interp, argv[1], mainwin);
+    tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[1]), mainwin);
     if (tkwin == NULL) {
 	return TCL_ERROR;
     }
-    if (argc == 2) {
+    if (objc == 2) {
 	other = NULL;
     } else {
-	other = Tk_NameToWindow(interp, argv[2], mainwin);
+	other = Tk_NameToWindow(interp, Tcl_GetString(objv[2]), mainwin);
 	if (other == NULL) {
 	    return TCL_ERROR;
 	}
     }
     if (Tk_RestackWindow(tkwin, Below, other) != TCL_OK) {
-	Tcl_AppendResult(interp, "can't lower \"", argv[1], "\" below \"",
-		argv[2], "\"", (char *) NULL);
+	Tcl_AppendResult(interp, "can't lower \"", Tcl_GetString(objv[1]),
+		"\" below \"", (other ? Tcl_GetString(objv[2]) : ""),
+		"\"", (char *) NULL);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -487,7 +489,7 @@ Tk_LowerCmd(clientData, interp, argc, argv)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_RaiseCmd --
+ * Tk_RaiseObjCmd --
  *
  *	This procedure is invoked to process the "raise" Tcl command.
  *	See the user documentation for details on what it does.
@@ -503,37 +505,37 @@ Tk_LowerCmd(clientData, interp, argc, argv)
 
 	/* ARGSUSED */
 int
-Tk_RaiseCmd(clientData, interp, argc, argv)
+Tk_RaiseObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Tk_Window mainwin = (Tk_Window) clientData;
     Tk_Window tkwin, other;
 
-    if ((argc != 2) && (argc != 3)) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " window ?aboveThis?\"", (char *) NULL);
+    if ((objc != 2) && (objc != 3)) {
+	Tcl_WrongNumArgs(interp, 1, objv, "window ?aboveThis?");
 	return TCL_ERROR;
     }
 
-    tkwin = Tk_NameToWindow(interp, argv[1], mainwin);
+    tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[1]), mainwin);
     if (tkwin == NULL) {
 	return TCL_ERROR;
     }
-    if (argc == 2) {
+    if (objc == 2) {
 	other = NULL;
     } else {
-	other = Tk_NameToWindow(interp, argv[2], mainwin);
+	other = Tk_NameToWindow(interp, Tcl_GetString(objv[2]), mainwin);
 	if (other == NULL) {
 	    return TCL_ERROR;
 	}
     }
     if (Tk_RestackWindow(tkwin, Above, other) != TCL_OK) {
-	Tcl_AppendResult(interp, "can't raise \"", argv[1], "\" above \"",
-		argv[2], "\"", (char *) NULL);
+	Tcl_AppendResult(interp, "can't raise \"", Tcl_GetString(objv[1]),
+		"\" above \"", (other ? Tcl_GetString(objv[2]) : ""),
+		"\"", (char *) NULL);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -566,10 +568,10 @@ Tk_TkObjCmd(clientData, interp, objc, objv)
     int index;
     Tk_Window tkwin;
     static char *optionStrings[] = {
-	"appname",	"scaling",	NULL
+	"appname",	"scaling",	"useinputmethods",	NULL
     };
     enum options {
-	TK_APPNAME,	TK_SCALING
+	TK_APPNAME,	TK_SCALING,	TK_USE_IM
     };
 
     tkwin = (Tk_Window) clientData;
@@ -598,14 +600,14 @@ Tk_TkObjCmd(clientData, interp, objc, objv)
 		string = Tcl_GetStringFromObj(objv[2], NULL);
 		winPtr->nameUid = Tk_GetUid(Tk_SetAppName(tkwin, string));
 	    }
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), winPtr->nameUid, -1);
+	    Tcl_AppendResult(interp, winPtr->nameUid, NULL);
 	    break;
 	}
 	case TK_SCALING: {
 	    Screen *screenPtr;
 	    int skip, width, height;
 	    double d;
-	    
+
 	    screenPtr = Tk_Screen(tkwin);
 
 	    skip = TkGetDisplayOf(interp, objc - 2, objv + 2, &tkwin);
@@ -618,7 +620,7 @@ Tk_TkObjCmd(clientData, interp, objc, objv)
 		d /= WidthMMOfScreen(screenPtr);
 		Tcl_SetDoubleObj(Tcl_GetObjResult(interp), d);
 	    } else if (objc - skip == 3) {
-		if (Tcl_GetDoubleFromObj(interp, objv[2 + skip], &d) != TCL_OK) {
+		if (Tcl_GetDoubleFromObj(interp, objv[2+skip], &d) != TCL_OK) {
 		    return TCL_ERROR;
 		}
 		d = (25.4 / 72) / d;
@@ -639,6 +641,40 @@ Tk_TkObjCmd(clientData, interp, objc, objv)
 	    }
 	    break;
 	}
+	case TK_USE_IM: {
+	    TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
+	    int skip;
+
+	    skip = TkGetDisplayOf(interp, objc-2, objv+2, &tkwin);
+	    if (skip < 0) {
+		return TCL_ERROR;
+	    } else if (skip) {
+		dispPtr = ((TkWindow *) tkwin)->dispPtr;
+	    }
+	    if ((objc - skip) == 3) {
+		/*
+		 * In the case where TK_USE_INPUT_METHODS is not defined,
+		 * this will be ignored and we will always return 0.
+		 * That will indicate to the user that input methods
+		 * are just not available.
+		 */
+		int bool;
+		if (Tcl_GetBooleanFromObj(interp, objv[2+skip], &bool)
+			!= TCL_OK) {
+		    return TCL_ERROR;
+		}
+#ifdef TK_USE_INPUT_METHODS
+		dispPtr->useInputMethods = bool;
+#endif /* TK_USE_INPUT_METHODS */
+	    } else if ((objc - skip) != 2) {
+		Tcl_WrongNumArgs(interp, 2, objv,
+			"?-displayof window? ?boolean?");
+		return TCL_ERROR;
+	    }
+	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp),
+		    dispPtr->useInputMethods);
+	    break;
+	}
     }
     return TCL_OK;
 }
@@ -800,7 +836,7 @@ WaitWindowProc(clientData, eventPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_UpdateCmd --
+ * Tk_UpdateObjCmd --
  *
  *	This procedure is invoked to process the "update" Tcl command.
  *	See the user documentation for details on what it does.
@@ -816,28 +852,27 @@ WaitWindowProc(clientData, eventPtr)
 
 	/* ARGSUSED */
 int
-Tk_UpdateCmd(clientData, interp, argc, argv)
+Tk_UpdateObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    int flags;
+    static char *updateOptions[] = {"idletasks", (char *) NULL};
+    int flags, index;
     TkDisplay *dispPtr;
 
-    if (argc == 1) {
+    if (objc == 1) {
 	flags = TCL_DONT_WAIT;
-    } else if (argc == 2) {
-	if (strncmp(argv[1], "idletasks", strlen(argv[1])) != 0) {
-	    Tcl_AppendResult(interp, "bad option \"", argv[1],
-		    "\": must be idletasks", (char *) NULL);
+    } else if (objc == 2) {
+	if (Tcl_GetIndexFromObj(interp, objv[1], updateOptions, "option", 0,
+		&index) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 	flags = TCL_IDLE_EVENTS;
     } else {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " ?idletasks?\"", (char *) NULL);
+        Tcl_WrongNumArgs(interp, 1, objv, "?idletasks?");
 	return TCL_ERROR;
     }
 
@@ -849,12 +884,12 @@ Tk_UpdateCmd(clientData, interp, argc, argv)
      * Thus, don't use any information from tkwin after calling
      * Tcl_DoOneEvent.
      */
-
+  
     while (1) {
 	while (Tcl_DoOneEvent(flags) != 0) {
 	    /* Empty loop body */
 	}
-	for (dispPtr = tkDisplayList; dispPtr != NULL;
+	for (dispPtr = TkGetDisplayList(); dispPtr != NULL;
 		dispPtr = dispPtr->nextPtr) {
 	    XSync(dispPtr->display, False);
 	}
@@ -898,10 +933,10 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     int index, x, y, width, height, useX, useY, class, skip;
-    char buf[128];
     char *string;
     TkWindow *winPtr;
     Tk_Window tkwin;
+    Tcl_Obj *resultPtr;
 
     static TkStateMap visualMap[] = {
 	{PseudoColor,	"pseudocolor"},
@@ -974,85 +1009,73 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	}
     }
     winPtr = (TkWindow *) tkwin;
+    resultPtr = Tcl_GetObjResult(interp);
 
     switch ((enum options) index) {
 	case WIN_CELLS: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    Tk_Visual(tkwin)->map_entries);
+	    Tcl_SetIntObj(resultPtr, Tk_Visual(tkwin)->map_entries);
 	    break;
 	}
 	case WIN_CHILDREN: {
 	    Tcl_Obj *strPtr;
 
-	    Tcl_ResetResult(interp);
 	    winPtr = winPtr->childList;
 	    for ( ; winPtr != NULL; winPtr = winPtr->nextPtr) {
 		strPtr = Tcl_NewStringObj(winPtr->pathName, -1);
-		Tcl_ListObjAppendElement(NULL,
-		     Tcl_GetObjResult(interp), strPtr);
+		Tcl_ListObjAppendElement(NULL, resultPtr, strPtr);
 	    }
 	    break;
 	}
 	case WIN_CLASS: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), Tk_Class(tkwin), -1);
+	    Tcl_SetStringObj(resultPtr, Tk_Class(tkwin), -1);
 	    break;
 	}
 	case WIN_COLORMAPFULL: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp),
+	    Tcl_SetBooleanObj(resultPtr,
 		    TkpCmapStressed(tkwin, Tk_Colormap(tkwin)));
 	    break;
 	}
 	case WIN_DEPTH: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_Depth(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_Depth(tkwin));
 	    break;
 	}
 	case WIN_GEOMETRY: {
-	    Tcl_ResetResult(interp);
+	    char buf[16 + TCL_INTEGER_SPACE * 4];
+
 	    sprintf(buf, "%dx%d+%d+%d", Tk_Width(tkwin), Tk_Height(tkwin),
 		    Tk_X(tkwin), Tk_Y(tkwin));
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), buf, -1);
+	    Tcl_SetStringObj(resultPtr, buf, -1);
 	    break;
 	}
 	case WIN_HEIGHT: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_Height(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_Height(tkwin));
 	    break;
 	}
 	case WIN_ID: {
+	    char buf[TCL_INTEGER_SPACE];
+	    
 	    Tk_MakeWindowExist(tkwin);
 	    TkpPrintWindowId(buf, Tk_WindowId(tkwin));
-	    Tcl_ResetResult(interp);
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), buf, -1);
+	    Tcl_SetStringObj(resultPtr, buf, -1);
 	    break;
 	}
 	case WIN_ISMAPPED: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp),
-		    (int) Tk_IsMapped(tkwin));
+	    Tcl_SetBooleanObj(resultPtr, (int) Tk_IsMapped(tkwin));
 	    break;
 	}
 	case WIN_MANAGER: {
-	    Tcl_ResetResult(interp);
 	    if (winPtr->geomMgrPtr != NULL) {
-		Tcl_SetStringObj(Tcl_GetObjResult(interp),
-		        winPtr->geomMgrPtr->name, -1);
+		Tcl_SetStringObj(resultPtr, winPtr->geomMgrPtr->name, -1);
 	    }
 	    break;
 	}
 	case WIN_NAME: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), Tk_Name(tkwin), -1);
+	    Tcl_SetStringObj(resultPtr, Tk_Name(tkwin), -1);
 	    break;
 	}
 	case WIN_PARENT: {
-	    Tcl_ResetResult(interp);
 	    if (winPtr->parentPtr != NULL) {
-		Tcl_SetStringObj(Tcl_GetObjResult(interp),
-		        winPtr->parentPtr->pathName, -1);
+		Tcl_SetStringObj(resultPtr, winPtr->parentPtr->pathName, -1);
 	    }
 	    break;
 	}
@@ -1078,80 +1101,66 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    } else {
 		TkGetPointerCoords((Tk_Window) winPtr, &x, &y);
 	    }
-	    Tcl_ResetResult(interp);
 	    if (useX & useY) {
+		char buf[TCL_INTEGER_SPACE * 2];
+		
 		sprintf(buf, "%d %d", x, y);
-		Tcl_SetStringObj(Tcl_GetObjResult(interp), buf, -1);
+		Tcl_SetStringObj(resultPtr, buf, -1);
 	    } else if (useX) {
-		Tcl_SetIntObj(Tcl_GetObjResult(interp), x);
+		Tcl_SetIntObj(resultPtr, x);
 	    } else {
-		Tcl_SetIntObj(Tcl_GetObjResult(interp), y);
+		Tcl_SetIntObj(resultPtr, y);
 	    }
 	    break;
 	}
 	case WIN_REQHEIGHT: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_ReqHeight(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_ReqHeight(tkwin));
 	    break;
 	}
 	case WIN_REQWIDTH: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_ReqWidth(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_ReqWidth(tkwin));
 	    break;
 	}
 	case WIN_ROOTX: {
 	    Tk_GetRootCoords(tkwin, &x, &y);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), x);
+	    Tcl_SetIntObj(resultPtr, x);
 	    break;
 	}
 	case WIN_ROOTY: {
 	    Tk_GetRootCoords(tkwin, &x, &y);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), y);
+	    Tcl_SetIntObj(resultPtr, y);
 	    break;
 	}
 	case WIN_SCREEN: {
+	    char buf[TCL_INTEGER_SPACE];
+	    
 	    sprintf(buf, "%d", Tk_ScreenNumber(tkwin));
-	    Tcl_ResetResult(interp);
-	    Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
-		    Tk_DisplayName(tkwin), ".", buf, NULL);
+	    Tcl_AppendStringsToObj(resultPtr, Tk_DisplayName(tkwin), ".",
+		    buf, NULL);
 	    break;
 	}
 	case WIN_SCREENCELLS: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    CellsOfScreen(Tk_Screen(tkwin)));
+	    Tcl_SetIntObj(resultPtr, CellsOfScreen(Tk_Screen(tkwin)));
 	    break;
 	}
 	case WIN_SCREENDEPTH: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    DefaultDepthOfScreen(Tk_Screen(tkwin)));
+	    Tcl_SetIntObj(resultPtr, DefaultDepthOfScreen(Tk_Screen(tkwin)));
 	    break;
 	}
 	case WIN_SCREENHEIGHT: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    HeightOfScreen(Tk_Screen(tkwin)));
+	    Tcl_SetIntObj(resultPtr, HeightOfScreen(Tk_Screen(tkwin)));
 	    break;
 	}
 	case WIN_SCREENWIDTH: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    WidthOfScreen(Tk_Screen(tkwin)));
+	    Tcl_SetIntObj(resultPtr, WidthOfScreen(Tk_Screen(tkwin)));
 	    break;
 	}
 	case WIN_SCREENMMHEIGHT: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    HeightMMOfScreen(Tk_Screen(tkwin)));
+	    Tcl_SetIntObj(resultPtr, HeightMMOfScreen(Tk_Screen(tkwin)));
 	    break;
 	}
 	case WIN_SCREENMMWIDTH: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp),
-		    WidthMMOfScreen(Tk_Screen(tkwin)));
+	    Tcl_SetIntObj(resultPtr, WidthMMOfScreen(Tk_Screen(tkwin)));
 	    break;
 	}
 	case WIN_SCREENVISUAL: {
@@ -1165,16 +1174,12 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	case WIN_TOPLEVEL: {
 	    winPtr = GetToplevel(tkwin);
 	    if (winPtr != NULL) {
-		Tcl_ResetResult(interp);
-		Tcl_SetStringObj(Tcl_GetObjResult(interp),
-			winPtr->pathName, -1);
+		Tcl_SetStringObj(resultPtr, winPtr->pathName, -1);
 	    }
 	    break;
 	}
 	case WIN_VIEWABLE: {
-	    int viewable;
-
-	    viewable = 0;
+	    int viewable = 0;
 	    for ( ; ; winPtr = winPtr->parentPtr) {
 		if ((winPtr == NULL) || !(winPtr->flags & TK_MAPPED)) {
 		    break;
@@ -1184,8 +1189,8 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 		    break;
 		}
 	    }
-	    Tcl_ResetResult(interp);
-	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp), viewable);
+
+	    Tcl_SetBooleanObj(resultPtr, viewable);
 	    break;
 	}
 	case WIN_VISUAL: {
@@ -1196,54 +1201,47 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    if (string == NULL) {
 		string = "unknown";
 	    }
-	    Tcl_ResetResult(interp);
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), string, -1);
+	    Tcl_SetStringObj(resultPtr, string, -1);
 	    break;
 	}
 	case WIN_VISUALID: {
-	    Tcl_ResetResult(interp);
+	    char buf[TCL_INTEGER_SPACE];
+
 	    sprintf(buf, "0x%x",
 		    (unsigned int) XVisualIDFromVisual(Tk_Visual(tkwin)));
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), buf, -1);
+	    Tcl_SetStringObj(resultPtr, buf, -1);
 	    break;
 	}
 	case WIN_VROOTHEIGHT: {
 	    Tk_GetVRootGeometry(tkwin, &x, &y, &width, &height);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), height);
+	    Tcl_SetIntObj(resultPtr, height);
 	    break;
 	}
 	case WIN_VROOTWIDTH: {
 	    Tk_GetVRootGeometry(tkwin, &x, &y, &width, &height);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), width);
+	    Tcl_SetIntObj(resultPtr, width);
 	    break;
 	}
 	case WIN_VROOTX: {
 	    Tk_GetVRootGeometry(tkwin, &x, &y, &width, &height);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), x);
+	    Tcl_SetIntObj(resultPtr, x);
 	    break;
 	}
 	case WIN_VROOTY: {
 	    Tk_GetVRootGeometry(tkwin, &x, &y, &width, &height);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), y);
+	    Tcl_SetIntObj(resultPtr, y);
 	    break;
 	}
 	case WIN_WIDTH: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_Width(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_Width(tkwin));
 	    break;
 	}
 	case WIN_X: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_X(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_X(tkwin));
 	    break;
 	}
 	case WIN_Y: {
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), Tk_Y(tkwin));
+	    Tcl_SetIntObj(resultPtr, Tk_Y(tkwin));
 	    break;
 	}
 
@@ -1262,9 +1260,7 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    }
 	    objv += skip;
 	    string = Tcl_GetStringFromObj(objv[2], NULL);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetLongObj(Tcl_GetObjResult(interp),
-		    (long) Tk_InternAtom(tkwin, string));
+	    Tcl_SetLongObj(resultPtr, (long) Tk_InternAtom(tkwin, string));
 	    break;
 	}
 	case WIN_ATOMNAME: {
@@ -1283,15 +1279,14 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    if (Tcl_GetLongFromObj(interp, objv[2], &id) != TCL_OK) {
 		return TCL_ERROR;
 	    }
-	    Tcl_ResetResult(interp);
 	    name = Tk_GetAtomName(tkwin, (Atom) id);
 	    if (strcmp(name, "?bad atom?") == 0) {
 		string = Tcl_GetStringFromObj(objv[2], NULL);
-		Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+		Tcl_AppendStringsToObj(resultPtr, 
 			"no atom exists with id \"", string, "\"", NULL);
 		return TCL_ERROR;
 	    }
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), name, -1);
+	    Tcl_SetStringObj(resultPtr, name, -1);
 	    break;
 	}
 	case WIN_CONTAINING: {
@@ -1315,9 +1310,7 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    }
 	    tkwin = Tk_CoordsToWindow(x, y, tkwin);
 	    if (tkwin != NULL) {
-		Tcl_ResetResult(interp);
-		Tcl_SetStringObj(Tcl_GetObjResult(interp),
-			Tk_PathName(tkwin), -1);
+		Tcl_SetStringObj(resultPtr, Tk_PathName(tkwin), -1);
 	    }
 	    break;
 	}
@@ -1354,9 +1347,7 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	            Tk_IdToWindow(Tk_Display(tkwin), (Window) id);
 	    if ((winPtr == NULL) ||
 		    (winPtr->mainPtr != ((TkWindow *) tkwin)->mainPtr)) {
-		Tcl_ResetResult(interp);
-		Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
-			"window id \"", string,
+		Tcl_AppendStringsToObj(resultPtr, "window id \"", string,
 			"\" doesn't exist in this application", (char *) NULL);
 		return TCL_ERROR;
 	    }
@@ -1369,9 +1360,7 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 
 	    tkwin = (Tk_Window) winPtr;
 	    if (Tk_PathName(tkwin) != NULL) {
-		Tcl_ResetResult(interp);
-		Tcl_SetStringObj(Tcl_GetObjResult(interp),
-		        Tk_PathName(tkwin), -1);
+		Tcl_SetStringObj(resultPtr, Tk_PathName(tkwin), -1);
 	    }
 	    break;
 	}
@@ -1389,12 +1378,14 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    }
 	    string = Tcl_GetStringFromObj(objv[2], NULL);
 	    winPtr = (TkWindow *) Tk_NameToWindow(interp, string, tkwin);
+	    Tcl_ResetResult(interp);
+	    resultPtr = Tcl_GetObjResult(interp);
+
 	    alive = 1;
 	    if ((winPtr == NULL) || (winPtr->flags & TK_ALREADY_DEAD)) {
 		alive = 0;
 	    }
-	    Tcl_ResetResult(interp); /* clear any error msg */
-	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp), alive);
+	    Tcl_SetBooleanObj(resultPtr, alive);
 	    break;
 	}
 	case WIN_FPIXELS: {
@@ -1414,9 +1405,8 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 		return TCL_ERROR;
 	    }
 	    pixels = mm * WidthOfScreen(Tk_Screen(tkwin))
-		/ WidthMMOfScreen(Tk_Screen(tkwin));
-	    Tcl_ResetResult(interp);
-	    Tcl_SetDoubleObj(Tcl_GetObjResult(interp), pixels);
+		    / WidthMMOfScreen(Tk_Screen(tkwin));
+	    Tcl_SetDoubleObj(resultPtr, pixels);
 	    break;
 	}
 	case WIN_PIXELS: {
@@ -1435,12 +1425,12 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    if (Tk_GetPixels(interp, tkwin, string, &pixels) != TCL_OK) {
 		return TCL_ERROR;
 	    }
-	    Tcl_ResetResult(interp);
-	    Tcl_SetIntObj(Tcl_GetObjResult(interp), pixels);
+	    Tcl_SetIntObj(resultPtr, pixels);
 	    break;
 	}
 	case WIN_RGB: {
 	    XColor *colorPtr;
+	    char buf[TCL_INTEGER_SPACE * 3];
 
 	    if (objc != 4) {
 		Tcl_WrongNumArgs(interp, 2, objv, "window colorName");
@@ -1459,16 +1449,16 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    sprintf(buf, "%d %d %d", colorPtr->red, colorPtr->green,
 		    colorPtr->blue);
 	    Tk_FreeColor(colorPtr);
-	    Tcl_ResetResult(interp);
-	    Tcl_SetStringObj(Tcl_GetObjResult(interp), buf, -1);
+	    Tcl_SetStringObj(resultPtr, buf, -1);
 	    break;
 	}
 	case WIN_VISUALSAVAILABLE: {
 	    XVisualInfo template, *visInfoPtr;
 	    int count, i;
-	    char visualIdString[16];
 	    int includeVisualId;
 	    Tcl_Obj *strPtr;
+	    char buf[16 + TCL_INTEGER_SPACE];
+	    char visualIdString[TCL_INTEGER_SPACE];
 
 	    if (objc == 3) {
 		includeVisualId = 0;
@@ -1490,9 +1480,8 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 	    template.screen = Tk_ScreenNumber(tkwin);
 	    visInfoPtr = XGetVisualInfo(Tk_Display(tkwin), VisualScreenMask,
 		    &template, &count);
-	    Tcl_ResetResult(interp);
 	    if (visInfoPtr == NULL) {
-		Tcl_SetStringObj(Tcl_GetObjResult(interp),
+		Tcl_SetStringObj(resultPtr,
 			"can't find any visuals for screen", -1);
 		return TCL_ERROR;
 	    }
@@ -1509,8 +1498,7 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 		    strcat(buf, visualIdString);
 		}
 		strPtr = Tcl_NewStringObj(buf, -1);
-		Tcl_ListObjAppendElement(NULL, Tcl_GetObjResult(interp),
-		        strPtr);
+		Tcl_ListObjAppendElement(NULL, resultPtr, strPtr);
 	    }
 	    XFree((char *) visInfoPtr);
 	    break;
@@ -1519,6 +1507,221 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
     return TCL_OK;
 }
 
+#if 0
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_WmObjCmd --
+ *
+ *	This procedure is invoked to process the "wm" Tcl command.
+ *	See the user documentation for details on what it does.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	See the user documentation.
+ *
+ *----------------------------------------------------------------------
+ */
+
+	/* ARGSUSED */
+int
+Tk_WmObjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* Main window associated with
+				 * interpreter. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
+{
+    Tk_Window tkwin;
+    TkWindow *winPtr;
+
+    static char *optionStrings[] = {
+	"aspect",	"client",	"command",	"deiconify",
+	"focusmodel",	"frame",	"geometry",	"grid",
+	"group",	"iconbitmap",	"iconify",	"iconmask",
+	"iconname",	"iconposition",	"iconwindow",	"maxsize",
+	"minsize",	"overrideredirect",	"positionfrom",	"protocol",
+	"resizable",	"sizefrom",	"state",	"title",
+	"tracing",	"transient",	"withdraw",	(char *) NULL
+    };
+    enum options {
+	TKWM_ASPECT,	TKWM_CLIENT,	TKWM_COMMAND,	TKWM_DEICONIFY,
+	TKWM_FOCUSMOD,	TKWM_FRAME,	TKWM_GEOMETRY,	TKWM_GRID,
+	TKWM_GROUP,	TKWM_ICONBMP,	TKWM_ICONIFY,	TKWM_ICONMASK,
+	TKWM_ICONNAME,	TKWM_ICONPOS,	TKWM_ICONWIN,	TKWM_MAXSIZE,
+	TKWM_MINSIZE,	TKWM_OVERRIDE,	TKWM_POSFROM,	TKWM_PROTOCOL,
+	TKWM_RESIZABLE,	TKWM_SIZEFROM,	TKWM_STATE,	TKWM_TITLE,
+	TKWM_TRACING,	TKWM_TRANSIENT,	TKWM_WITHDRAW
+    };
+
+    tkwin = (Tk_Window) clientData;
+
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "option window ?arg?");
+	return TCL_ERROR;
+    }
+    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, "option", 0,
+	    &index) != TCL_OK) {
+	return TCL_ERROR;
+    }
+
+    if (index == TKWM_TRACING) {
+	TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
+
+	if ((objc != 2) && (objc != 3)) {
+	    Tcl_WrongNumArgs(interp, 1, objv, "tracing ?boolean?");
+	    return TCL_ERROR;
+	}
+	if (objc == 2) {
+	    Tcl_SetObjResult(interp, Tcl_NewBooleanObj(dispPtr->wmTracing));
+	    return TCL_OK;
+	}
+	return Tcl_GetBooleanFromObj(interp, objv[2], &dispPtr->wmTracing);
+    }
+
+    if (objc < 3) {
+	Tcl_WrongNumArgs(interp, 2, objv, "window ?arg?");
+	return TCL_ERROR;
+    }
+
+    winPtr = (TkWindow *) Tk_NameToWindow(interp,
+	    Tcl_GetString(objv[2]), tkwin);
+    if (winPtr == NULL) {
+	return TCL_ERROR;
+    }
+    if (!(winPtr->flags & TK_TOP_LEVEL)) {
+	Tcl_AppendResult(interp, "window \"", winPtr->pathName,
+		"\" isn't a top-level window", (char *) NULL);
+	return TCL_ERROR;
+    }
+
+    switch ((enum options) index) {
+	case TKWM_ASPECT: {
+	    TkpWmAspectCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_CLIENT: {
+	    TkpWmClientCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_COMMAND: {
+	    TkpWmCommandCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_DEICONIFY: {
+	    TkpWmDeiconifyCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_FOCUSMOD: {
+	    TkpWmFocusmodCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_FRAME: {
+	    TkpWmFrameCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_GEOMETRY: {
+	    TkpWmGeometryCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_GRID: {
+	    TkpWmGridCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_GROUP: {
+	    TkpWmGroupCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_ICONBMP: {
+	    TkpWmIconbitmapCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_ICONIFY: {
+	    TkpWmIconifyCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_ICONMASK: {
+	    TkpWmIconmaskCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_ICONNAME: {
+	    /* slight Unix variation */
+	    TkpWmIconnameCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_ICONPOS: {
+	    /* nearly same - 1 line more on Unix */
+	    TkpWmIconpositionCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_ICONWIN: {
+	    TkpWmIconwindowCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_MAXSIZE: {
+	    /* nearly same, win diffs */
+	    TkpWmMaxsizeCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_MINSIZE: {
+	    /* nearly same, win diffs */
+	    TkpWmMinsizeCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_OVERRIDE: {
+	    /* almost same */
+	    TkpWmOverrideCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_POSFROM: {
+	    /* Equal across platforms */
+	    TkpWmPositionfromCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_PROTOCOL: {
+	    /* Equal across platforms */
+	    TkpWmProtocolCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_RESIZABLE: {
+	    /* almost same */
+	    TkpWmResizableCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_SIZEFROM: {
+	    /* Equal across platforms */
+	    TkpWmSizefromCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_STATE: {
+	    TkpWmStateCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_TITLE: {
+	    TkpWmTitleCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_TRANSIENT: {
+	    TkpWmTransientCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+	case TKWM_WITHDRAW: {
+	    TkpWmWithdrawCmd(interp, tkwin, winPtr, objc, objv);
+	    break;
+	}
+    }
+
+    updateGeom:
+    if (!(wmPtr->flags & (WM_UPDATE_PENDING|WM_NEVER_MAPPED))) {
+	Tcl_DoWhenIdle(UpdateGeometryInfo, (ClientData) winPtr);
+	wmPtr->flags |= WM_UPDATE_PENDING;
+    }
+    return TCL_OK;
+}
+#endif
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1567,7 +1770,8 @@ TkGetDisplayOf(interp, objc, objv, tkwinPtr)
 	return 0;
     }
     string = Tcl_GetStringFromObj(objv[0], &length);
-    if ((length >= 2) && (strncmp(string, "-displayof", (unsigned) length) == 0)) {
+    if ((length >= 2) &&
+	    (strncmp(string, "-displayof", (unsigned) length) == 0)) {
         if (objc < 2) {
 	    Tcl_SetStringObj(Tcl_GetObjResult(interp),
 		    "value for \"-displayof\" missing", -1);
@@ -1647,3 +1851,5 @@ GetToplevel(tkwin)
      }
      return winPtr;
 }
+
+
