@@ -6,12 +6,12 @@
  *	and Tcl commands.
  *
  * Copyright (c) 1990-1993 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkSelect.c,v 1.8 1999/01/26 04:11:26 jingham Exp $
+ * RCS: @(#) $Id: tkSelect.c,v 1.7.6.1 2000/05/04 21:26:24 spolk Exp $
  */
 
 #include "tkInt.h"
@@ -26,6 +26,11 @@
 typedef struct {
     Tcl_Interp *interp;		/* Interpreter in which to invoke command. */
     int cmdLength;		/* # of non-NULL bytes in command. */
+    int charOffset;		/* The offset of the next char to retrieve. */
+    int byteOffset;		/* The expected byte offset of the next
+				 * chunk. */
+    char buffer[TCL_UTF_MAX];	/* A buffer to hold part of a UTF character
+				 * that is split across chunks.*/
     char command[4];		/* Command to invoke.  Actual space is
 				 * allocated as large as necessary.  This
 				 * must be the last entry in the structure. */
@@ -45,12 +50,16 @@ typedef struct LostCommand {
 } LostCommand;
 
 /*
- * Shared variables:
+ * The structure below is used to keep each thread's pending list
+ * separate.
  */
 
-TkSelInProgress *pendingPtr = NULL;
+typedef struct ThreadSpecificData {
+    TkSelInProgress *pendingPtr;
 				/* Topmost search in progress, or
 				 * NULL if none. */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Forward declarations for procedures defined in this file:
@@ -199,6 +208,8 @@ Tk_DeleteSelHandler(tkwin, selection, target)
     TkWindow *winPtr = (TkWindow *) tkwin;
     register TkSelHandler *selPtr, *prevPtr;
     register TkSelInProgress *ipPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Find the selection handler to be deleted, or return if it doesn't
@@ -220,7 +231,8 @@ Tk_DeleteSelHandler(tkwin, selection, target)
      * handler is dead.
      */
 
-    for (ipPtr = pendingPtr; ipPtr != NULL; ipPtr = ipPtr->nextPtr) {
+    for (ipPtr = tsdPtr->pendingPtr; ipPtr != NULL; 
+            ipPtr = ipPtr->nextPtr) {
 	if (ipPtr->selPtr == selPtr) {
 	    ipPtr->selPtr = NULL;
 	}
@@ -236,7 +248,12 @@ Tk_DeleteSelHandler(tkwin, selection, target)
 	prevPtr->nextPtr = selPtr->nextPtr;
     }
     if (selPtr->proc == HandleTclCommand) {
-	ckfree((char *) selPtr->clientData);
+	/*
+	 * Mark the CommandInfo as deleted and free it if we can.
+	 */
+
+	((CommandInfo*)selPtr->clientData)->interp = NULL;
+	Tcl_EventuallyFree(selPtr->clientData, Tcl_Free);
     }
     ckfree((char *) selPtr);
 }
@@ -431,7 +448,7 @@ Tk_ClearSelection(tkwin, selection)
  * Results:
  *	The return value is a standard Tcl return value.
  *	If an error occurs (such as no selection exists)
- *	then an error message is left in interp->result.
+ *	then an error message is left in the interp's result.
  *
  * Side effects:
  *	The standard X11 protocols are used to retrieve the
@@ -457,7 +474,7 @@ Tk_ClearSelection(tkwin, selection)
  *	the "portion" arguments in separate calls will contain
  *	successive parts of the selection.  Proc should normally
  *	return TCL_OK.  If it detects an error then it should return
- *	TCL_ERROR and leave an error message in interp->result; the
+ *	TCL_ERROR and leave an error message in the interp's result; the
  *	remainder of the selection retrieval will be aborted.
  *
  *--------------------------------------------------------------
@@ -480,6 +497,8 @@ Tk_GetSelection(interp, tkwin, selection, target, proc, clientData)
     TkWindow *winPtr = (TkWindow *) tkwin;
     TkDisplay *dispPtr = winPtr->dispPtr;
     TkSelectionInfo *infoPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (dispPtr->multipleAtom == None) {
 	TkSelInit(tkwin);
@@ -528,13 +547,13 @@ Tk_GetSelection(interp, tkwin, selection, target, proc, clientData)
 	    offset = 0;
 	    result = TCL_OK;
 	    ip.selPtr = selPtr;
-	    ip.nextPtr = pendingPtr;
-	    pendingPtr = &ip;
+	    ip.nextPtr = tsdPtr->pendingPtr;
+	    tsdPtr->pendingPtr = &ip;
 	    while (1) {
 		count = (selPtr->proc)(selPtr->clientData, offset, buffer,
 			TK_SEL_BYTES_AT_ONCE);
 		if ((count < 0) || (ip.selPtr == NULL)) {
-		    pendingPtr = ip.nextPtr;
+		    tsdPtr->pendingPtr = ip.nextPtr;
 		    goto cantget;
 		}
 		if (count > TK_SEL_BYTES_AT_ONCE) {
@@ -548,7 +567,7 @@ Tk_GetSelection(interp, tkwin, selection, target, proc, clientData)
 		}
 		offset += count;
 	    }
-	    pendingPtr = ip.nextPtr;
+	    tsdPtr->pendingPtr = ip.nextPtr;
 	}
 	return result;
     }
@@ -602,9 +621,8 @@ Tk_SelectionCmd(clientData, interp, argc, argv)
     char **args;
 
     if (argc < 2) {
-	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option ?arg arg ...?\"",
-		argv[0]);
+	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+		" option ?arg arg ...?\"", (char *) NULL);
 	return TCL_ERROR;
     }
     c = argv[1][0];
@@ -785,6 +803,9 @@ Tk_SelectionCmd(clientData, interp, argc, argv)
 	    cmdInfoPtr = (CommandInfo *) ckalloc((unsigned) (
 		    sizeof(CommandInfo) - 3 + cmdLength));
 	    cmdInfoPtr->interp = interp;
+	    cmdInfoPtr->charOffset = 0;
+	    cmdInfoPtr->byteOffset = 0;
+	    cmdInfoPtr->buffer[0] = '\0';
 	    cmdInfoPtr->cmdLength = cmdLength;
 	    strcpy(cmdInfoPtr->command, args[1]);
 	    Tk_CreateSelHandler(tkwin, selection, target, HandleTclCommand,
@@ -854,7 +875,7 @@ Tk_SelectionCmd(clientData, interp, argc, argv)
 
 	    if ((infoPtr != NULL)
 		    && (infoPtr->owner != winPtr->dispPtr->clipWindow)) {
-		interp->result = Tk_PathName(infoPtr->owner);
+		Tcl_SetResult(interp, Tk_PathName(infoPtr->owner), TCL_STATIC);
 	    }
 	    return TCL_OK;
 	}
@@ -878,13 +899,66 @@ Tk_SelectionCmd(clientData, interp, argc, argv)
 	Tk_OwnSelection(tkwin, selection, LostSelection, (ClientData) lostPtr);
 	return TCL_OK;
     } else {
-	sprintf(interp->result,
-		"bad option \"%.50s\": must be clear, get, handle, or own",
-		argv[1]);
+	Tcl_AppendResult(interp, "bad option \"", argv[1],
+		"\": must be clear, get, handle, or own", (char *) NULL);
 	return TCL_ERROR;
     }
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkSelGetInProgress --
+ *
+ *	This procedure returns a pointer to the thread-local
+ *      list of pending searches.
+ *
+ * Results:
+ *	The return value is a pointer to the first search in progress, 
+ *      or NULL if there are none. 
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+TkSelInProgress *
+TkSelGetInProgress _ANSI_ARGS_((void))
+{
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+    return tsdPtr->pendingPtr;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkSelSetInProgress --
+ *
+ *	This procedure is used to set the thread-local list of pending 
+ *      searches.  It is required because the pending list is kept
+ *      in thread local storage.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+void
+TkSelSetInProgress(pendingPtr)
+    TkSelInProgress *pendingPtr;
+{
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+   tsdPtr->pendingPtr = pendingPtr;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -909,6 +983,8 @@ TkSelDeadWindow(winPtr)
     register TkSelHandler *selPtr;
     register TkSelInProgress *ipPtr;
     TkSelectionInfo *infoPtr, *prevPtr, *nextPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * While deleting all the handlers, be careful to check whether
@@ -919,13 +995,19 @@ TkSelDeadWindow(winPtr)
     while (winPtr->selHandlerList != NULL) {
 	selPtr = winPtr->selHandlerList;
 	winPtr->selHandlerList = selPtr->nextPtr;
-	for (ipPtr = pendingPtr; ipPtr != NULL; ipPtr = ipPtr->nextPtr) {
+	for (ipPtr = tsdPtr->pendingPtr; ipPtr != NULL; 
+                ipPtr = ipPtr->nextPtr) {
 	    if (ipPtr->selPtr == selPtr) {
 		ipPtr->selPtr = NULL;
 	    }
 	}
 	if (selPtr->proc == HandleTclCommand) {
-	    ckfree((char *) selPtr->clientData);
+	    /*
+	     * Mark the CommandInfo as deleted and free it if we can.
+	     */
+
+	    ((CommandInfo*)selPtr->clientData)->interp = NULL;
+	    Tcl_EventuallyFree(selPtr->clientData, Tcl_Free);
 	}
 	ckfree((char *) selPtr);
     }
@@ -1120,20 +1202,41 @@ HandleTclCommand(clientData, offset, buffer, maxBytes)
     int spaceNeeded, length;
 #define MAX_STATIC_SIZE 100
     char staticSpace[MAX_STATIC_SIZE];
-    char *command;
-    Tcl_Interp *interp;
+    char *command, *string;
+    Tcl_Interp *interp = cmdInfoPtr->interp;
     Tcl_DString oldResult;
+    Tcl_Obj *objPtr;
+    int extraBytes, charOffset, count, numChars;
+    char *p;
 
     /*
-     * We must copy the interpreter pointer from CommandInfo because the
-     * command could delete the handler, freeing the CommandInfo data before we
-     * are done using it. We must also protect the interpreter from being
-     * deleted too soo.
+     * We must also protect the interpreter and the command from being
+     * deleted too soon.
      */
 
-    interp = cmdInfoPtr->interp;
+    Tcl_Preserve(clientData);
     Tcl_Preserve((ClientData) interp);
 
+    /*
+     * Compute the proper byte offset in the case where the last chunk
+     * split a character.
+     */
+
+    if (offset == cmdInfoPtr->byteOffset) {
+	charOffset = cmdInfoPtr->charOffset;
+	extraBytes = strlen(cmdInfoPtr->buffer);
+	if (extraBytes > 0) {
+	    strcpy(buffer, cmdInfoPtr->buffer);
+	    maxBytes -= extraBytes;
+	    buffer += extraBytes;
+	}
+    } else {
+	cmdInfoPtr->byteOffset = 0;
+	cmdInfoPtr->charOffset = 0;
+	extraBytes = 0;
+	charOffset = 0;
+    }
+
     /*
      * First, generate a command by taking the command string
      * and appending the offset and maximum # of bytes.
@@ -1145,7 +1248,7 @@ HandleTclCommand(clientData, offset, buffer, maxBytes)
     } else {
 	command = (char *) ckalloc((unsigned) spaceNeeded);
     }
-    sprintf(command, "%s %d %d", cmdInfoPtr->command, offset, maxBytes);
+    sprintf(command, "%s %d %d", cmdInfoPtr->command, charOffset, maxBytes);
 
     /*
      * Execute the command.  Be sure to restore the state of the
@@ -1155,14 +1258,41 @@ HandleTclCommand(clientData, offset, buffer, maxBytes)
     Tcl_DStringInit(&oldResult);
     Tcl_DStringGetResult(interp, &oldResult);
     if (TkCopyAndGlobalEval(interp, command) == TCL_OK) {
-	length = strlen(interp->result);
-	if (length > maxBytes) {
-	    length = maxBytes;
+	objPtr = Tcl_GetObjResult(interp);
+	string = Tcl_GetStringFromObj(objPtr, &length);
+	count = (length > maxBytes) ? maxBytes : length;
+	memcpy((VOID *) buffer, (VOID *) string, (size_t) count);
+	buffer[count] = '\0';
+
+	/*
+	 * Update the partial character information for the next
+	 * retrieval if the command has not been deleted.
+	 */
+
+	if (cmdInfoPtr->interp != NULL) {
+	    if (length <= maxBytes) {
+		cmdInfoPtr->charOffset += Tcl_NumUtfChars(string, -1);
+		cmdInfoPtr->buffer[0] = '\0';
+	    } else {
+		p = string;
+		string += count;
+		numChars = 0;
+		while (p < string) {
+		    p = Tcl_UtfNext(p);
+		    numChars++;
+		}
+		cmdInfoPtr->charOffset += numChars;
+		length = p - string;
+		if (length > 0) {
+		    strncpy(cmdInfoPtr->buffer, string, (size_t) length);
+		}
+		cmdInfoPtr->buffer[length] = '\0';
+	    }		
+	    cmdInfoPtr->byteOffset += count + extraBytes;
 	}
-	memcpy((VOID *) buffer, (VOID *) interp->result, (size_t) length);
-	buffer[length] = '\0';
+	count += extraBytes;
     } else {
-	length = -1;
+	count = -1;
     }
     Tcl_DStringResult(interp, &oldResult);
 
@@ -1170,8 +1300,10 @@ HandleTclCommand(clientData, offset, buffer, maxBytes)
 	ckfree(command);
     }
 
+
+    Tcl_Release(clientData);
     Tcl_Release((ClientData) interp);
-    return length;
+    return count;
 }
 
 /*
@@ -1299,11 +1431,10 @@ TkSelDefaultSelection(infoPtr, target, buffer, maxBytes, typePtr)
 
 static void
 LostSelection(clientData)
-    ClientData clientData;		/* Pointer to CommandInfo structure. */
+    ClientData clientData;		/* Pointer to LostCommand structure. */
 {
     LostCommand *lostPtr = (LostCommand *) clientData;
-    char *oldResultString;
-    Tcl_FreeProc *oldFreeProc;
+    Tcl_Obj *objPtr;
     Tcl_Interp *interp;
 
     interp = lostPtr->interp;
@@ -1314,22 +1445,16 @@ LostSelection(clientData)
      * restore it after executing the command.
      */
 
-    oldFreeProc = interp->freeProc;
-    if (oldFreeProc != TCL_STATIC) {
-	oldResultString = interp->result;
-    } else {
-	oldResultString = (char *) ckalloc((unsigned)
-		(strlen(interp->result) + 1));
-	strcpy(oldResultString, interp->result);
-	oldFreeProc = TCL_DYNAMIC;
-    }
-    interp->freeProc = TCL_STATIC;
+    objPtr = Tcl_GetObjResult(interp);
+    Tcl_IncrRefCount(objPtr);
+    Tcl_ResetResult(interp);
+
     if (TkCopyAndGlobalEval(interp, lostPtr->command) != TCL_OK) {
 	Tcl_BackgroundError(interp);
     }
-    Tcl_FreeResult(interp);
-    interp->result = oldResultString;
-    interp->freeProc = oldFreeProc;
+
+    Tcl_SetObjResult(interp, objPtr);
+    Tcl_DecrRefCount(objPtr);
 
     Tcl_Release((ClientData) interp);
     
@@ -1339,3 +1464,4 @@ LostSelection(clientData)
 
     ckfree((char *) lostPtr);
 }
+
