@@ -10,23 +10,21 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixInt.h,v 1.7 1999/01/26 04:11:42 jingham Exp $
+ * RCS: @(#) $Id: tkUnixInt.h,v 1.7.6.1 2000/05/04 21:26:30 spolk Exp $
  */
 
 #ifndef _TKUNIXINT
 #define _TKUNIXINT
 
+#ifndef _TKINT
+#include "tkInt.h"
+#endif
+
 /*
  * Prototypes for procedures that are referenced in files other
  * than the ones they're defined in.
  */
-
-EXTERN void		TkCreateXEventSource _ANSI_ARGS_((void));
-EXTERN TkWindow *	TkpGetContainer _ANSI_ARGS_((TkWindow *embeddedPtr));
-EXTERN TkWindow *	TkpGetWrapperWindow _ANSI_ARGS_((TkWindow *winPtr));
-EXTERN Window		TkUnixContainerId _ANSI_ARGS_((TkWindow *winPtr));
-EXTERN int		TkUnixDoOneXEvent _ANSI_ARGS_((Tcl_Time *timePtr));
-EXTERN void		TkUnixSetMenubar _ANSI_ARGS_((Tk_Window tkwin,
-				Tk_Window menubar));
+#include "tkIntPlatDecls.h"
 
 #endif /* _TKUNIXINT */
+
